@@ -375,7 +375,8 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
 
 
     if(CMulticalendar::MCInstance()->getSystemTimeShift() != (*event_iter)->getTzOffset() &&
-       (*event_iter)->getAllDay())
+       ((*event_iter)->getAllDay() ||
+	(*event_iter)->getType() == E_TODO))
     {
       event_end_time += event_time_offset - system_time_offset;
       event_start_time += event_time_offset - system_time_offset;
@@ -383,11 +384,15 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
 
     struct tm* time = localtime(&event_start_time);
 
-    if(earliest_update_end_time>event_end_time)
-      earliest_update_end_time = end_time+60;
+    if(event_end_time > date_today_time && earliest_update_end_time>event_end_time)
+    {
+      earliest_update_end_time = event_end_time+60;
+    }
     if(date_today_time < event_start_time &&
        earliest_update_end_time > event_start_time)
+    {
       earliest_update_end_time = event_start_time+60;
+    }
     char buf[255];
     if((*event_iter)->getType() == E_TODO)
     {
@@ -461,63 +466,6 @@ cal_put_age_on_summary(const int& born,
   return new_summary;
 }
 
-static void
-cal_read_todos_for_calendar(CCalendar* calendar,
-			    time_t start_time,
-			    time_t end_time,
-			    SortedCComponentSet& all_components,
-			    CalHomePlugin* desktop_plugin)
-{
-  int error = 0;
-  int offset = 0;
-  int size = 0;
-  int limit = 200;
-  struct tm today_midnight; 
-  cal_home_time_today_midnight(&today_midnight);
-
-  do
-  {
-    vector<CComponent*> calendars_components = calendar->getComponents(E_TODO,
-								       start_time,
-								       end_time,
-								       limit,
-								       offset,
-								       error);
-    size = calendars_components.size();
-    offset+=200;
-    limit+=200;
-    time_t last_start_date = start_time;
-    for(int k=0;k<calendars_components.size();++k)
-    {
-      CComponent* component = calendars_components[k];
-      gchar buf[256];
-      time_t event_time = component->getDateStart();
-      struct tm* time = localtime(&event_time);
-      if(_todo_is_old(component, &today_midnight))
-      {
-	if((component->getStatus() == 0) &&
-	   (desktop_plugin->show_old_undone_tasks))
-	{
-	  all_components.insert(component);
-	}
-	else
-	{
-	  delete component;
-	}
-      }
-      else if((component->getStatus() == 0) ||
-	      desktop_plugin->show_done_tasks)
-      {
-	all_components.insert(component);
-      }
-      else
-      {
-	delete component;
-      }
-    }
-  }while(size != 0);
-}
-
 static vector<CComponent*> 
 calendar_getComponents(int calendarId,
 		       int type, int iStDate,
@@ -548,7 +496,7 @@ calendar_getComponents(int calendarId,
 		    "select * from %s where "
 		    "%s = %d  AND  %s = %d AND (%s + %s) < %d AND (%s + %s) > %d AND (%s != %d AND %s != %d) "
 		    "union select * from %s where "
-		    "%s = %d AND  %s = %d  AND (%s = 2  OR allday = 1) AND (%s + %s)= %d AND (%s != %d AND %s != %d) " 
+		    "%s = %d AND  %s = %d  AND (%s = 2  OR allday = 1) AND (%s + %s)>= %d AND (%s + %s)< %d AND (%s != %d AND %s != %d) " 
 		    "union select * from %s where "
 		    "%s = %d AND %s = %d  AND allday = 0 AND (%s = %d OR %s = %d) AND (%d < %s OR %d = %s) "
 		    "union select * from %s where "
@@ -559,8 +507,9 @@ calendar_getComponents(int calendarId,
 		    ENTRY_FIELD_FLAGS, HAS_RECURRENCE,ENTRY_FIELD_FLAGS, HAS_RECURRENCE_ALARM,
 		    
 		    COMPONENTS_TABLE,ENTRY_FIELD_CALID, calendarId, ENTRY_FIELD_TYPE, type, ENTRY_FIELD_TYPE,
-		    ENTRY_FIELD_DTSTART, ENTRY_FIELD_TZOFFSET, iNewStDate,ENTRY_FIELD_FLAGS, 
-		    HAS_RECURRENCE,ENTRY_FIELD_FLAGS, HAS_RECURRENCE_ALARM, 
+		    ENTRY_FIELD_DTSTART, ENTRY_FIELD_TZOFFSET, iNewStDate, 
+		    ENTRY_FIELD_DTSTART, ENTRY_FIELD_TZOFFSET, iNewEndDate,
+		    ENTRY_FIELD_FLAGS, HAS_RECURRENCE,ENTRY_FIELD_FLAGS, HAS_RECURRENCE_ALARM, 
 		    
 		    COMPONENTS_TABLE,ENTRY_FIELD_CALID, calendarId, ENTRY_FIELD_TYPE, type,
 		    ENTRY_FIELD_FLAGS, HAS_RECURRENCE,ENTRY_FIELD_FLAGS, HAS_RECURRENCE_ALARM,iStDate, 
@@ -694,6 +643,64 @@ calendar_getComponents(int calendarId,
   return vListEntry;
 }
 
+static void
+cal_read_todos_for_calendar(CCalendar* calendar,
+			    time_t start_time,
+			    time_t end_time,
+			    SortedCComponentSet& all_components,
+			    CalHomePlugin* desktop_plugin)
+{
+  int error = 0;
+  int offset = 0;
+  int size = 0;
+  int limit = 200;
+  struct tm today_midnight; 
+  cal_home_time_today_midnight(&today_midnight);
+  do
+  {
+    vector<CComponent*> calendars_components = calendar_getComponents(calendar->getCalendarId(),
+								      E_TODO,
+								      start_time, 
+								      end_time,
+								      limit,
+								      offset,
+								      error);
+
+    size = calendars_components.size();
+    offset+=200;
+    limit+=200;
+    time_t last_start_date = start_time;
+    for(int k=0;k<calendars_components.size();++k)
+    {
+      CComponent* component = calendars_components[k];
+      gchar buf[256];
+      time_t event_time = component->getDateStart();
+      struct tm* time = localtime(&event_time);
+      if(_todo_is_old(component, &today_midnight))
+      {
+	if((component->getStatus() == 0) &&
+	   (desktop_plugin->show_old_undone_tasks))
+	{
+	  all_components.insert(component);
+	}
+	else
+	{
+	  delete component;
+	}
+      }
+      else if((component->getStatus() == 0) ||
+	      desktop_plugin->show_done_tasks)
+      {
+	all_components.insert(component);
+      }
+      else
+      {
+	delete component;
+      }
+    }
+  }while(size != 0);
+}
+
 static void
 cal_read_ccomponents_for_calendar(CCalendar* calendar,
 				  time_t start_time,
@@ -824,7 +831,7 @@ cal_read_ccomponents_all_visible(SortedCComponentSet& all_components,
 	{
 	  cal_read_todos_for_calendar(cal,
 				      -1,
-				      -1,
+				      end_time,
 				      all_components,
 				      desktop_plugin);
 	}
